@@ -1,4 +1,4 @@
-# Copyright (C) 2021-2023, Mindee.
+# Copyright (C) 2021-2024, Mindee.
 
 # This program is licensed under the Apache License 2.0.
 # See LICENSE or go to <https://opensource.org/licenses/Apache-2.0> for full license details.
@@ -59,7 +59,7 @@ class _AbstractDataset:
             # Conditions to assess it is detection model with multiple classes and avoid confusion with other tasks.
             if (
                 isinstance(target, dict)
-                and all([isinstance(item, np.ndarray) for item in target.values()])
+                and all(isinstance(item, np.ndarray) for item in target.values())
                 and set(target.keys()) != {"boxes", "labels"}  # avoid confusion with obj detection target
             ):
                 img_transformed = _copy_tensor(img)
@@ -82,6 +82,7 @@ class _VisionDataset(_AbstractDataset):
     """Implements an abstract dataset
 
     Args:
+    ----
         url: URL of the dataset
         file_name: name of the file once downloaded
         file_hash: expected SHA256 of the file
