@@ -1,7 +1,7 @@
-# Copyright (C) 2021-2022, Mindee.
+# Copyright (C) 2021-2024, Mindee.
 
-# This program is licensed under the Apache License version 2.
-# See LICENSE or go to <https://www.apache.org/licenses/LICENSE-2.0.txt> for full license details.
+# This program is licensed under the Apache License 2.0.
+# See LICENSE or go to <https://opensource.org/licenses/Apache-2.0> for full license details.
 
 import math
 from typing import Any, List, Tuple, Union
@@ -15,13 +15,14 @@ from torchvision.transforms import transforms as T
 from doctr.transforms import Resize
 from doctr.utils.multithreading import multithread_exec
 
-__all__ = ['PreProcessor']
+__all__ = ["PreProcessor"]
 
 
 class PreProcessor(nn.Module):
     """Implements an abstract preprocessor object which performs casting, resizing, batching and normalization.
 
     Args:
+    ----
         output_size: expected size of each page in format (H, W)
         batch_size: the size of page batches
         mean: mean value of the training distribution by channel
@@ -32,9 +33,8 @@ class PreProcessor(nn.Module):
         self,
         output_size: Tuple[int, int],
         batch_size: int,
-        mean: Tuple[float, float, float] = (.5, .5, .5),
-        std: Tuple[float, float, float] = (1., 1., 1.),
-        fp16: bool = False,
+        mean: Tuple[float, float, float] = (0.5, 0.5, 0.5),
+        std: Tuple[float, float, float] = (1.0, 1.0, 1.0),
         **kwargs: Any,
     ) -> None:
         super().__init__()
@@ -43,22 +43,20 @@ class PreProcessor(nn.Module):
         # Perform the division by 255 at the same time
         self.normalize = T.Normalize(mean, std)
 
-    def batch_inputs(
-        self,
-        samples: List[torch.Tensor]
-    ) -> List[torch.Tensor]:
+    def batch_inputs(self, samples: List[torch.Tensor]) -> List[torch.Tensor]:
         """Gather samples into batches for inference purposes
 
         Args:
+        ----
             samples: list of samples of shape (C, H, W)
 
         Returns:
+        -------
             list of batched samples (*, C, H, W)
         """
-
         num_batches = int(math.ceil(len(samples) / self.batch_size))
         batches = [
-            torch.stack(samples[idx * self.batch_size: min((idx + 1) * self.batch_size, len(samples))], dim=0)
+            torch.stack(samples[idx * self.batch_size : min((idx + 1) * self.batch_size, len(samples))], dim=0)
             for idx in range(int(num_batches))
         ]
 
@@ -77,23 +75,23 @@ class PreProcessor(nn.Module):
         x = self.resize(x)
         # Data type
         if x.dtype == torch.uint8:
-            x = x.to(dtype=torch.float32).div(255).clip(0, 1)
-        x = x.to(dtype=torch.float32)
+            x = x.to(dtype=torch.float32).div(255).clip(0, 1)  # type: ignore[union-attr]
+        else:
+            x = x.to(dtype=torch.float32)  # type: ignore[union-attr]
 
-        return x
+        return x  # type: ignore[return-value]
 
-    def __call__(
-        self,
-        x: Union[torch.Tensor, np.ndarray, List[Union[torch.Tensor, np.ndarray]]]
-    ) -> List[torch.Tensor]:
+    def __call__(self, x: Union[torch.Tensor, np.ndarray, List[Union[torch.Tensor, np.ndarray]]]) -> List[torch.Tensor]:
         """Prepare document data for model forwarding
 
         Args:
+        ----
             x: list of images (np.array) or tensors (already resized and batched)
+
         Returns:
+        -------
             list of page batches
         """
-
         # Input type check
         if isinstance(x, (np.ndarray, torch.Tensor)):
             if x.ndim != 4:
@@ -105,23 +103,26 @@ class PreProcessor(nn.Module):
             elif x.dtype not in (torch.uint8, torch.float16, torch.float32):
                 raise TypeError("unsupported data type for torch.Tensor")
             # Resizing
-            if x.shape[-2] != self.resize.size[0] or x.shape[-1] != self.resize.size[1]:
-                x = F.resize(x, self.resize.size, interpolation=self.resize.interpolation)
+            if x.shape[-2] != self.resize.size[0] or x.shape[-1] != self.resize.size[1]:  # type: ignore[union-attr]
+                x = F.resize(
+                    x, self.resize.size, interpolation=self.resize.interpolation, antialias=self.resize.antialias
+                )
             # Data type
-            if x.dtype == torch.uint8:
-                x = x.to(dtype=torch.float32).div(255).clip(0, 1)
-            x = x.to(dtype=torch.float32)
+            if x.dtype == torch.uint8:  # type: ignore[union-attr]
+                x = x.to(dtype=torch.float32).div(255).clip(0, 1)  # type: ignore[union-attr]
+            else:
+                x = x.to(dtype=torch.float32)  # type: ignore[union-attr]
             batches = [x]
 
         elif isinstance(x, list) and all(isinstance(sample, (np.ndarray, torch.Tensor)) for sample in x):
             # Sample transform (to tensor, resize)
-            samples = multithread_exec(self.sample_transforms, x)
+            samples = list(multithread_exec(self.sample_transforms, x))
             # Batching
-            batches = self.batch_inputs(samples)  # type: ignore[arg-type]
+            batches = self.batch_inputs(samples)  # type: ignore[assignment]
         else:
             raise TypeError(f"invalid input type: {type(x)}")
 
         # Batch transforms (normalize)
-        batches = multithread_exec(self.normalize, batches)  # type: ignore[assignment]
+        batches = list(multithread_exec(self.normalize, batches))
 
-        return batches
+        return batches  # type: ignore[return-value]
