@@ -1,12 +1,12 @@
-# Copyright (C) 2021-2022, Mindee.
+# Copyright (C) 2021-2024, Mindee.
 
-# This program is licensed under the Apache License version 2.
-# See LICENSE or go to <https://www.apache.org/licenses/LICENSE-2.0.txt> for full license details.
+# This program is licensed under the Apache License 2.0.
+# See LICENSE or go to <https://opensource.org/licenses/Apache-2.0> for full license details.
 
 from pathlib import Path
 from typing import List, Tuple, Union
 
-__all__ = ['Point2D', 'BoundingBox', 'Polygon4P', 'Polygon', 'Bbox']
+__all__ = ["Point2D", "BoundingBox", "Polygon4P", "Polygon", "Bbox"]
 
 
 Point2D = Tuple[float, float]
